@@ -7,7 +7,7 @@ FIREWALLCMD="firewall-cmd"
 FIREWALLCMD_DIRECT="${FIREWALLCMD} --direct"
 FIREWALLCMD_QUIET="${FIREWALLCMD} --quiet"
 FIREWALLCMD_QUIET_DIRECT="${FIREWALLCMD_QUIET} --direct"
-IPTABLES=iptables
+IPTABLES="iptables -w"
 
 aembit_agent_proxy_user_name=aembit_agent_proxy
 aembit_group_id=$(id --group "${aembit_agent_proxy_user_name}")
@@ -23,7 +23,7 @@ aembit_nat_OUTPUT=aembit_nat_OUTPUT
 aembit_nat_PREROUTING=aembit_nat_PREROUTING
 aembit_filter_INPUT=aembit_filter_INPUT
 
-agent_proxy_notification_endpoint="localhost:${AEMBIT_HEALTH_CHECK_PORT}/iptables/loaded"
+agent_proxy_notification_endpoint="localhost:${AEMBIT_SERVICE_PORT}/iptables/loaded"
 
 # Create all aembit rule chains.
 create_all_aembit_chains() {
@@ -143,6 +143,15 @@ function fwd_direct_query_ipv4_rules() {
     ${FIREWALLCMD_DIRECT} --get-rules ipv4 "${table}" "${chain}"
 }
 
+# Formats a firewalld rule for deletion.
+#
+# firewall-d output format doesn't include the table or chain at the beginning of the rule,
+# but does include a leading '0', so we need to remove it and prepend the table and chain.
+function format_fwd_delete_rule() {
+    local rule=$1
+    echo "nat ${aembit_nat_OUTPUT} ${rule#0 }"
+}
+
 # Iptables - Add ipv4 chain
 # args: table, chain name
 function iptables_add_ipv4_chain() {
@@ -158,7 +167,7 @@ function iptables_add_ipv4_rule() {
     local chain=$2
     shift
     shift
-    ${IPTABLES} -t "${table}" -A "${chain}" -w "$@"
+    ${IPTABLES} -t "${table}" -A "${chain}" "$@"
 }
 
 # Remove ipv4 rule using iptables
@@ -168,7 +177,7 @@ function iptables_remove_ipv4_rule() {
     local chain=$2
     shift
     shift
-    ${IPTABLES} -t "${table}" -D "${chain}" -w "$@"
+    ${IPTABLES} -t "${table}" -D "${chain}" "$@"
 }
 
 # Remove ipv4 chain using iptables
@@ -190,8 +199,19 @@ function iptables_query_ipv4_rules() {
     ${IPTABLES} -t "${table}" -S "${chain}"
 }
 
+# Formats an iptables rule for deletion.
+#
+# iptables removal expects "<table> <chain> [...]", but the format output by iptables -S is "-A <chain> [...]"
+function format_iptables_delete_rule() {
+    local rule=$1
+    echo "${rule//-A /nat }"
+}
+
 function query_ipv4_rules() {
-    if firewalld_is_active_and_running; then
+    # squelch the output, we just need the status
+    _fwd_output=$(firewalld_is_active_and_running)
+    fwd_status=$?
+    if [ "$fwd_status" -eq 0 ]; then
         fwd_direct_query_ipv4_rules "$@"
     else
         iptables_query_ipv4_rules "$@"
@@ -230,6 +250,20 @@ function remove_ipv4_rule() {
     fi
 }
 
+# Formats a firewall rule for deletion.
+function format_delete_rule() {
+    local rule=$1
+    # squelch the output, we just need the status
+    _fwd_output=$(firewalld_is_active_and_running)
+    fwd_status=$?
+    if [ "$fwd_status" -eq 0 ]; then
+        format_fwd_delete_rule "${rule}"
+    else
+        format_iptables_delete_rule "${rule}"
+    fi
+
+}
+
 function save_and_reload_daemon() {
     if firewalld_is_active_and_running; then
 	    # save firewalld rules and reload the daemon
@@ -279,19 +313,62 @@ function redirect_outbound_tcp_traffic_to_ap() {
     fi
 }
 
-# Manipulates the outbound traffic rules to redirect traffic to the Agent Proxy.
+# Removes a list of firewall rules from the firewall.
 #
-# This function expects arguments to be:
-# $1: hostname,
+# This function expects arguments to be a list of rules (as strings), like:
+# remove_ipv4_rules "rule1" "rule2" "rule3" ...
+function remove_ipv4_rules() {
+    active_rules_for_host=("$@")
+    for rule in "${active_rules_for_host[@]}"; do
+        rule=$(format_delete_rule "${rule}")
+        echo "removing: ${rule}"
+        IFS=' ' read -ra rule_parts <<< "${rule}"
+        remove_ipv4_rule "${rule_parts[@]}"
+    done
+}
+
+# Adds a list of firewall rules to the firewall.
+#
+# Args:
+# $1: The table to add the rule to,
+# $2: The hostname to add as the rule's comment.
+# $@: The list of rules to add.
+function add_ipv4_rules() {
+    for ip in "${incoming_ips_for_host[@]}"; do
+
+        if [[ "${table}" == "${aembit_nat_PREROUTING}" ]] && [[ -n "${container_cidr}" ]]; then
+            # add a source IP to the rule if we're dealing with the PREROUTING table and we have a container CIDR
+            rule_parts=("nat" "${table}" "-d" "${ip}" "-p" "tcp" "--syn" "-s" "${container_cidr}" "-m" "comment" "--comment" "${hostname}" "-j" "REDIRECT" "--to-port" "${agent_proxy_port}")
+        else
+            rule_parts=("nat" "${table}" "-d" "${ip}" "-p" "tcp" "--syn" "-m" "comment" "--comment" "${hostname}" "-j" "REDIRECT" "--to-port" "${agent_proxy_port}")
+        fi
+
+        echo "adding: ${rule_parts[*]}"
+        add_ipv4_rule "${rule_parts[@]}"
+    done
+}
+
+
+# Manipulates outbound traffic rules to redirect traffic to the Agent Proxy.
+#
+# Args:
+# $1: the firewall table to manipulate,
+# $2: the hostname to redirect traffic for,
 # $@: list of IPs which the hostname resolved to.
-function redirect_outbound_tcp_traffic_for_ips_to_ap() {
-    hostname="$1"
+function redirect_outbound_tcp_traffic() {
+    table="$1"
+    hostname="$2"
+    shift
     shift
     incoming_ips_for_host=("$@")
 
-    active_rules=$(query_ipv4_rules nat "${aembit_nat_OUTPUT}" | grep -e "-m comment --comment \"\?${hostname}\"\?")
+    active_rules=$(query_ipv4_rules nat "${table}" | grep -e "-m comment --comment \"\?${hostname}\"\?")
     # Remove all quotes from the active rules (otherwise we end up double-quoting comments in the rules, which breaks the comparison below)
     active_rules=${active_rules//\"}
+
+    echo -e "active rules:\n${active_rules}"
+    echo -e "incoming IPs: ${incoming_ips_for_host[*]}"
+
     IFS=$'\n' read -rd '' -a active_rules_for_host <<< "${active_rules}"
 
     # We need to do three things:
@@ -319,33 +396,35 @@ function redirect_outbound_tcp_traffic_for_ips_to_ap() {
         done
     done
 
-    # Finally, remove and add rules
-    for rule in "${active_rules_for_host[@]}"; do
-        if firewalld_is_active_and_running; then
-            # fwd output format doesn't include the table or chain at the beginning of the rule, but does include a leading '0',
-            # so we need to remove it and prepend the table and chain
-            rule="nat ${aembit_nat_OUTPUT} ${rule#0 }"
-        else
-            # iptables removal expects "<table> <chain> [...]", but the iptables output format is "-A <chain> [...]"
-            rule="${rule//-A /nat }"
-        fi
+    remove_ipv4_rules "${active_rules_for_host[@]}"
 
-        echo "removing: ${rule}"
-        IFS=' ' read -ra rule_parts <<< "${rule}"
-        remove_ipv4_rule "${rule_parts[@]}"
-    done
+    add_ipv4_rules "${table}" "${hostname}" "${incoming_ips_for_host[@]}"
+}
 
-    for ip in "${incoming_ips_for_host[@]}"; do
-        rule_parts=("nat" "${aembit_nat_OUTPUT}" "-d" "${ip}" "-p" "tcp" "--syn" "-m" "comment" "--comment" "${hostname}" "-j" "REDIRECT" "--to-port" "${agent_proxy_port}")
-        echo "adding: ${rule_parts[*]}"
-        add_ipv4_rule "${rule_parts[@]}"
-    done
+# Manipulates the outbound traffic rules to redirect traffic to the Agent Proxy.
+#
+# Args:
+# $1: hostname,
+# $@: list of IPs which the hostname resolved to.
+function redirect_outbound_tcp_traffic_for_ips_to_ap() {
+    hostname="$1"
+    shift
+
+    # redirect host-based traffic
+    redirect_outbound_tcp_traffic "${aembit_nat_OUTPUT}" "${hostname}" "$@"
+
+    # also redirect container-based traffic if Agent Proxy was installed with a container CIDR
+    if [ -n "$container_cidr" ]; then
+        redirect_outbound_tcp_traffic "${aembit_nat_PREROUTING}" "${hostname}" "$@"
+    fi
 }
 
 # Redirect TCP traffic from containers to the AP.
 function redirect_container_traffic_to_ap() {
-    # REDIRECT tcp traffic from container CIDR to the AP
-    add_ipv4_rule nat "${aembit_nat_PREROUTING}" -p tcp --syn -s "${container_cidr}" -j REDIRECT --to-port "${agent_proxy_port}"
+    if [ -z "${AEMBIT_STEERING_ALLOWED_HOSTS}" ]; then
+        # REDIRECT tcp traffic from container CIDR to the AP if we didn't specify custom steering
+        add_ipv4_rule nat "${aembit_nat_PREROUTING}" -p tcp --syn -s "${container_cidr}" -j REDIRECT --to-port "${agent_proxy_port}"
+    fi
 }
 
 # TCP related rules.
@@ -369,8 +448,8 @@ function tcp_proxy_rules() {
     fi
 }
 
+# Notify Agent Proxy that firewall rules were installed.
 function notify_ap() {
-    # Notify Agent Proxy that firewall rules were installed.
     max_attempts=10
 
     for ((i=1; i<=max_attempts; i++)); do
@@ -380,14 +459,13 @@ function notify_ap() {
             break
         else
             echo "Couldn't notify Agent Proxy, received HTTP $status_code ($i/$max_attempts)."
-            sleep 2
+            sleep 1
         fi
     done
 
     if ((i > max_attempts)); then
         echo "Couldn't notify Agent Proxy after $max_attempts attempts."
     fi
-    
 }
 
 # Install chain rules for steering traffic to the Agent Proxy.
